;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;																			;
;	Abwaertszaehler, der vier Stellen einer 7Segment-Anzeige mit			;
;	gemeinsamer Anode herunterzaehlt(50 HZ).								;
;	Die maximale einstellbare Zeit ist 99 Min und 59 Sek					;
;	Es kann der Startwert beim ersten Durchlauf vorher eingestellt werden.	;
;	Der eingestellte Defaultwert ist 40 sek									;
;	PC5->Voreinstellung	der Sekunden(Taster schaltet L an PIN)				;
;	PC4->Voreinstellung der Minuten (Taster schaltet L an PIN)				;
;	PC3->Start des Abwaertszaehlers	(Taster schaltet L an PIN)				;
;	PC0-PC2 muessen L-Pegel anliegen haben									;
;	PB5->Ausgang zur Ansteuerung des Relais(H -> Relais an)					;
;	PB4->gemeinsame Anode Einerstelle der Sekunden							;
;	PB3->gemeinsame Anode Zehnerstelle der Sekunden							;
;	PB2->gemeinsame Anode Einerstelle der Minuten							;
;	PB1->gemeinsame Anode Zehnerstelle der Minuten							;
;	PortD->Ansteuerung 7Segment-Anzeige										;
;																			;
;	Der Sekundentakt wird ueber Timer-Interrupt erzeugt.					;
;																			;
;	 by www.thelastinstance.de				Kontakt:SWOD@thelastinstance.de	;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.include "m8def.inc"

;Registern zur besseren Uebersicht Namen geben
.def null = r0
.def eins = r1
.def zwei = r2
.def drei = r3
.def vier = r4
.def fuenf = r5
.def sechs = r6
.def sieben = r7
.def acht = r8
.def neun = r9
.def puffer_sekeiner = r10
.def puffer_sekzehner = r11
.def puffer_mineiner = r12
.def puffer_minzehner = r13
.def eingabe = r17
.def timerup = r18		;Pause beim Voreinstellen der Zeit
.def secount = r19
.def sekeiner = r20
.def sekzehner = r21
.def mineiner = r22
.def minzehner = r23
.def neustart = r24
.def intreg = r25
;ENDE

;Konstante festlegen
;.equ tm1end = 40000	;Timer1 Endwert(entspricht 100
.equ tm1end = 18432	;Timer1 Endwert(entspricht 100
					;Interrupts/s oder Zykluszeit 10ms)
;ENDE

;Interrupt-Vektoren festlegen
.cseg				;Codesegment	
.org 0x00			;ab Adresse 0x00 im Programm-Speicher
rjmp start			;reset
reti				;extint0
reti				;extint1
reti				;timer2_comp
reti				;timer2_ovf
reti				;timer1_capt
rjmp t1int			;timer1_compA alle 10msec
reti				;timer1_compB
reti				;timer1_ovf
reti				;timer0_ovf
reti				;spi_transfer_complete
reti				;usart_rx_complete
reti				;usart_udr_empty
reti				;usart_tx_complete
reti				;adc_conversion_complete
reti				;eeprom_ready
reti				;analog_comp
reti				;two-wire_serial_interface
reti				;store_program_memory_ready
;ENDE

;Initialisierung
start:
		;Register mit 7Segment-Ausgabe laden
		ldi r16, 0x40
		mov null, r16
		ldi r16, 0x79
		mov eins, r16
		ldi r16, 0x24
		mov zwei, r16
		ldi r16, 0x30
		mov drei, r16
		ldi r16, 0x19
		mov vier, r16
		ldi r16, 0x12
		mov fuenf, r16
		ldi r16, 0x02
		mov sechs, r16
		ldi r16, 0x78
		mov sieben, r16
		ldi r16, 0x00
		mov acht, r16
		ldi r16, 0x10
		mov neun, r16
		;ENDE

		;Stackpointer initialisieren fr Befehl 'rcall'
		ldi r16, LOW(RAMEND)			; LOW-Byte der obersten RAM-Adresse
		out SPL, r16
		ldi r16, HIGH(RAMEND)			; HIGH-Byte der obersten RAM-Adresse
		out SPH, r16
				
		clr secount			;Sekundenzhler auf 0 setzen
		clr neustart		;Neustart-Indikator auf 0 setzen
		ldi r16, 0xFF
		out DDRD, r16		;PortD als Ausgang konfigurieren
		out DDRB, r16		;PortB als Ausgang konfigurieren
		ldi r16, 0x00
		out DDRC, r16		;PortC als Eingang konfigurieren
		out PORTB, r16

		;Referenzwerte laden
		ldi timerup, 0x19
		ldi sekeiner, 0x00
		ldi sekzehner, 0x04		;Voreingestellt fr 40 sek
		ldi mineiner, 0x00
		ldi minzehner, 0x00
		;ENDE
;ENDE

;Eingaberoutine
input:
		cli						;Clear Global Interrupt
		rcall minausgabelinks	;Sprung zur Ausgabeprozedur
		in eingabe, PINC	;an PortC anliegende Werte einlesen
		cpi eingabe, 0x18	;wenn Taster PC5 betaetigt(L an PC5|H an PC4|H an PC3)
		breq sekaufwaerts
		cpi eingabe, 0x28	;wenn Taster PC4 betaetigt(H an PC5|L an PC4|H an PC3)
		breq minaufwaerts
		cpi eingabe, 0x08	;Taster PC4+PC5 betaetigt(L an PC5|L an PC4|H an PC3)
		breq beideaufwaerts
		cpi eingabe, 0x30	;wenn Taster PC3 betaetigt(H an PC5|H an PC4|L an PC3)
		breq premain

		rjmp input
;ENDE

;Vorbereitung zum Abwaertszaehlen
premain:
		mov puffer_sekeiner, sekeiner			;eingestellten Startwert puffern
		mov puffer_sekzehner, sekzehner			;eingestellten Startwert puffern
		mov puffer_mineiner, mineiner			;eingestellten Startwert puffern
		mov puffer_minzehner, minzehner			;eingestellten Startwert puffern
		wdr					;Watchdog zurcksetzen
		ldi r16, 0b0001000	;Watchdog aktivieren, 15ms
		out wdtcr, r16		;ausgeben

		ldi r16, high(tm1end);Timer1 Endwert1 H
		out ocr1ah, r16		;ausgeben
		ldi r16, low(tm1end);Timer Endwert1 L
		out ocr1al, r16		;ausgeben
		clr r16
		out tccr1a, r16		;Timer1 kein PWM, kein Comp-Out
		ldi r16, (1<<ctc1)+(1<<cs10);Timer1 lschen nach Vergleich
		out tccr1b, r16		;und Vorteiler=1 setzen
		
		clr r16
		out gimsk, r16		;externe Interrupts sperren
		ldi r16, 1<<ocie1a	
		out timsk, r16		;Timer1 Vergleich1 freigeben

		sei					;Set Global Interrupt	
		sbi PORTB, 1		;Relaisausgang auf H schalten	
;ENDE

;Hauptroutine
main:
		cpi neustart, 0x01
		breq input
		rcall minausgabelinks	;Sprung zur Ausgabeprozedur
		rjmp main		
;ENDE

;Zaehlalgorithmus zur Voreinstellung des Startwertes der Sekunden
sekaufwaerts:
		tst timerup
		breq sekaufwaerts_einer
		dec timerup

		rjmp input

sekaufwaerts_einer:
		ldi timerup, 0x19
		cpi sekeiner, 0x09
		breq sekaufwaerts_zehner
		inc sekeiner

		rjmp input

sekaufwaerts_zehner:
		ldi sekeiner, 0x00
		cpi sekzehner, 0x05
		breq sekanfang
		inc sekzehner
		
		rjmp input

sekanfang:
		ldi sekzehner, 0x00
		
		rjmp input
;Ende

;Zaehlalgorithmus zur Voreinstellung des Startwertes der Minuten
minaufwaerts:
		tst timerup
		breq minaufwaerts_einer
		dec timerup

		rjmp input

minaufwaerts_einer:
		ldi timerup, 0x19
		cpi mineiner, 0x09
		breq minaufwaerts_zehner
		inc mineiner

		rjmp input

minaufwaerts_zehner:
		ldi mineiner, 0x00
		cpi minzehner, 0x09
		breq minanfang
		inc minzehner
		
		rjmp input

minanfang:
		ldi minzehner, 0x00
		
		rjmp input
;Ende

;Zaehlalgorithmus zur Voreinstellung des Startwertes der Minuten und Sekunden
beideaufwaerts:
		tst timerup
		breq sprung
		dec timerup

		rjmp input

sprung:
		rcall beideaufwaerts_einer

		rjmp input
;Ende

;Timer1 Interruptroutine
t1int:
		in intreg, sreg		;CPU-Flags sichern
		wdr					;Watchdog zuruecksetzen
		inc secount			;Sekundenzaehler um eins erhoehen
		cpi secount, 0x64	;wenn eine Sekunde vorbei ist
		breq checksekeiner	;dann Sprung nach checksekeiner
		out sreg, intreg	;CPU-Flags wiederherstellen
		reti

checksekeiner:
		tst sekeiner
		breq checksekzehner
		dec sekeiner
		out sreg, intreg	;CPU-Flags wiederherstellen
		clr secount			;Sekundenzaehler auf 0 setzen
		reti

checksekzehner:
		tst sekzehner
		breq checkmineiner
		dec sekzehner
		ldi sekeiner, 0x09
		out sreg, intreg	;CPU-Flags wiederherstellen
		clr secount			;Sekundenzaehler auf 0 setzen
		reti

checkmineiner:
		tst mineiner
		breq checkminzehner
		dec mineiner
		ldi sekzehner, 0x05
		ldi sekeiner, 0x09
		out sreg, intreg	;CPU-Flags wiederherstellen
		clr secount			;Sekundenzaehler auf 0 setzen
		reti

checkminzehner:
		tst minzehner
		breq ende
		dec minzehner
		ldi mineiner, 0x09
		ldi sekzehner, 0x05
		ldi sekeiner, 0x09
		out sreg, intreg	;CPU-Flags wiederherstellen
		clr secount			;Sekundenzaehler auf 0 setzen
		reti

ende:
		cbi PORTB, 1		;Relais abschalten				
		ldi neustart, 0x01	;Neustart auf 1 setzen	
		mov sekeiner, puffer_sekeiner			;auf letzten Startwert zurcksetzen
		mov sekzehner, puffer_sekzehner			;auf letzten Startwert zurcksetzen
		mov mineiner, puffer_mineiner			;auf letzten Startwert zurcksetzen
		mov minzehner, puffer_minzehner			;auf letzten Startwert zurcksetzen
		out sreg, intreg	;CPU-Flags wiederherstellen
		clr secount			;Sekundenzaehler auf 0 setzen
		reti
;ENDE

;Anzeige der Zehnerstelle fuer Voreinstellung der Minuten
minausgabelinks:
		sbi PORTB, 2		;linke Ziffer aktivieren
		cpi minzehner, 0x09
		breq minausgabelneun
		cpi minzehner, 0x08
		breq minausgabelacht
		cpi minzehner, 0x07
		breq minausgabelsieben
		cpi minzehner, 0x06
		breq minausgabelsechs
		cpi minzehner, 0x05
		breq minausgabelfuenf
		cpi minzehner, 0x04
		breq minausgabelvier
		cpi minzehner, 0x03
		breq minausgabeldrei
		cpi minzehner, 0x02
		breq minausgabelzwei
		cpi minzehner, 0x01
		breq minausgabeleins
		cpi minzehner, 0x00
		breq minausgabelnull

minausgabelneun:
		out PORTD, neun		;Ausgabe linke Ziffer 9
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabelacht:
		out PORTD, acht		;Ausgabe linke Ziffer 8
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabelsieben:
		out PORTD, sieben	;Ausgabe linke Ziffer 7
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabelsechs:
		out PORTD, sechs	;Ausgabe linke Ziffer 6
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabelfuenf:
		out PORTD, fuenf	;Ausgabe linke Ziffer 5
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts
		
minausgabelvier:
		out PORTD, vier		;Ausgabe linke Ziffer 4
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts	
	
minausgabeldrei:
		out PORTD, drei		;Ausgabe linke Ziffer 3
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabelzwei:
		out PORTD, zwei		;Ausgabe linke Ziffer 2
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabeleins:
		out PORTD, eins		;Ausgabe linke Ziffer 1
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts

minausgabelnull:
		out PORTD, null		;Ausgabe linke Ziffer 0
		rcall pause
		cbi PORTB, 2		;linke Ziffer deaktivieren	
		
		rjmp minausgaberechts
;ENDE

;Anzeige der Einerstelle fuer Voreinstellung der Sekunden
minausgaberechts:
		sbi PORTB, 3		;rechte Ziffer aktivieren
		cpi mineiner, 0x09
		breq minausgaberneun
		cpi mineiner, 0x08
		breq minausgaberacht
		cpi mineiner, 0x07
		breq minausgabersieben
		cpi mineiner, 0x06
		breq minausgabersechs
		cpi mineiner, 0x05
		breq minausgaberfuenf
		cpi mineiner, 0x04
		breq minausgabervier
		cpi mineiner, 0x03
		breq minausgaberdrei
		cpi mineiner, 0x02
		breq minausgaberzwei
		cpi mineiner, 0x01
		breq minausgabereins
		cpi mineiner, 0x00
		breq minausgabernull

minausgaberneun:
		out PORTD, neun		;Ausgabe rechte Ziffer 9	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgaberacht:
		out PORTD, acht		;Ausgabe rechte Ziffer 8	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgabersieben:
		out PORTD, sieben	;Ausgabe rechte Ziffer 7	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgabersechs:
		out PORTD, sechs	;Ausgabe rechte Ziffer 6	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgaberfuenf:
		out PORTD, fuenf	;Ausgabe rechte Ziffer 5	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgabervier:
		out PORTD, vier		;Ausgabe rechte Ziffer 4	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgaberdrei:
		out PORTD, drei		;Ausgabe rechte Ziffer 3	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgaberzwei:
		out PORTD, zwei		;Ausgabe rechte Ziffer 2	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgabereins:
		out PORTD, eins		;Ausgabe rechte Ziffer 1	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks

minausgabernull:
		out PORTD, null		;Ausgabe rechte Ziffer 0	
		rcall pause
		cbi PORTB, 3		;rechte Ziffer deaktivieren
		
		rjmp sekausgabelinks
;ENDE

;Anzeige der Zehnerstelle fuer Voreinstellung der Sekunden
sekausgabelinks:
		sbi PORTB, 4		;linke Ziffer aktivieren
		cpi sekzehner, 0x05
		breq sekausgabelfuenf
		cpi sekzehner, 0x04
		breq sekausgabelvier
		cpi sekzehner, 0x03
		breq sekausgabeldrei
		cpi sekzehner, 0x02
		breq sekausgabelzwei
		cpi sekzehner, 0x01
		breq sekausgabeleins
		cpi sekzehner, 0x00
		breq sekausgabelnull

sekausgabelfuenf:
		out PORTD, fuenf	;Ausgabe linke Ziffer 5
		rcall pause
		cbi PORTB, 4		;linke Ziffer deaktivieren	
		
		rjmp sekausgaberechts
		
sekausgabelvier:
		out PORTD, vier		;Ausgabe linke Ziffer 4
		rcall pause
		cbi PORTB, 4		;linke Ziffer deaktivieren	
		
		rjmp sekausgaberechts	
	
sekausgabeldrei:
		out PORTD, drei		;Ausgabe linke Ziffer 3
		rcall pause
		cbi PORTB, 4		;linke Ziffer deaktivieren	
		
		rjmp sekausgaberechts

sekausgabelzwei:
		out PORTD, zwei		;Ausgabe linke Ziffer 2
		rcall pause
		cbi PORTB, 4		;linke Ziffer deaktivieren	
		
		rjmp sekausgaberechts

sekausgabeleins:
		out PORTD, eins		;Ausgabe linke Ziffer 1
		rcall pause
		cbi PORTB, 4		;linke Ziffer deaktivieren	
		
		rjmp sekausgaberechts

sekausgabelnull:
		out PORTD, null		;Ausgabe linke Ziffer 0
		rcall pause
		cbi PORTB, 4		;linke Ziffer deaktivieren	
		
		rjmp sekausgaberechts
;ENDE

;Anzeige der Einerstelle fuer Voreinstellung der Sekunden
sekausgaberechts:
		sbi PORTB, 5		;rechte Ziffer aktivieren
		cpi sekeiner, 0x09
		breq sekausgaberneun
		cpi sekeiner, 0x08
		breq sekausgaberacht
		cpi sekeiner, 0x07
		breq sekausgabersieben
		cpi sekeiner, 0x06
		breq sekausgabersechs
		cpi sekeiner, 0x05
		breq sekausgaberfuenf
		cpi sekeiner, 0x04
		breq sekausgabervier
		cpi sekeiner, 0x03
		breq sekausgaberdrei
		cpi sekeiner, 0x02
		breq sekausgaberzwei
		cpi sekeiner, 0x01
		breq sekausgabereins
		cpi sekeiner, 0x00
		breq sekausgabernull

sekausgaberneun:
		out PORTD, neun		;Ausgabe rechte Ziffer 9	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret


sekausgaberacht:
		out PORTD, acht		;Ausgabe rechte Ziffer 8	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgabersieben:
		out PORTD, sieben	;Ausgabe rechte Ziffer 7	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgabersechs:
		out PORTD, sechs	;Ausgabe rechte Ziffer 6	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgaberfuenf:
		out PORTD, fuenf	;Ausgabe rechte Ziffer 5	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgabervier:
		out PORTD, vier		;Ausgabe rechte Ziffer 4	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgaberdrei:
		out PORTD, drei		;Ausgabe rechte Ziffer 3	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgaberzwei:
		out PORTD, zwei		;Ausgabe rechte Ziffer 2	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgabereins:
		out PORTD, eins		;Ausgabe rechte Ziffer 1	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret

sekausgabernull:
		out PORTD, null		;Ausgabe rechte Ziffer 0	
		rcall pause
		cbi PORTB, 5		;rechte Ziffer deaktivieren
		ret
;ENDE

;Aufwaertszaehlen Minuten und Sekunden
beideaufwaerts_einer:
		ldi timerup, 0x19
		cpi mineiner, 0x09
		breq beidemin_zehner
		inc mineiner
		cpi sekeiner, 0x09
		breq beidesek_zehner
		inc sekeiner

		ret

beidemin_zehner:
		ldi mineiner, 0x00
		cpi minzehner, 0x09
		breq beideminanfang
		inc minzehner
		cpi sekeiner, 0x09
		breq beidesek_zehner
		inc sekeiner
		
		ret

beideminanfang:
		ldi minzehner, 0x00
		cpi sekeiner, 0x09
		breq beidesek_zehner
		inc sekeiner		

		ret

beidesek_zehner:
		ldi sekeiner, 0x00
		cpi sekzehner, 0x05
		breq beidesekanfang
		inc sekzehner
		
		ret

beidesekanfang:
		ldi sekzehner, 0x00
		
		ret
;ENDE

;Pausenschleife 5ms fuer Anzeige
pause:
		ldi r30, 0x11
loop0:	ldi r31, 0xC9
loop1:	dec r31
		brne loop1
		dec r30
		brne loop0
		ret					;Ruecksprung zu rcall pause
;ENDE

;pause: ret
